#!/usr/bin/env python

from xml.dom import minidom
import os
import sys

from pymisc import *

import states
from fold import foldl,foldr

parseMap = dict ()
def parserAdd(obj):
	parseMap["%s:%s" % (obj._ns, obj._name)] = obj
def parser(el):
	return parseMap.get("%s:%s" % (el.namespaceURI, el.localName)) or Undef

"""
Debug
def pr(x): print x
def pA(o):
        print "Print Attr %s" % o
        map(lambda x: pr((x, getattr(o, x))), o._attr)
"""

class xBase:
	_ns = "http://psha.org.ru/ns/mnet"
	_prefix = "mnet"
	_name = None
        _attr = []

	def __init__(self, xml = None):
		self.value = None
		self.attrs = dict()
		self.childs = dict()
                map (lambda x: setattr(self, x, None), self._attr)
		if xml:
			self.parse(xml)
	
	def xml(self):
		"""
		Simple Text dump
		"""
                map(lambda x: self.attrs.update({x:getattr(self, x, None)}), self._attr)
		n = self.Element(self._name, self.value)
		if not n.attributes:
			n.attriutes = dict()
		map (lambda (a, v): v is None or n.attributes.__setitem__(a, str(v)), self.attrs.items())
		map (lambda l: map (lambda e: e and n.appendChild(e.xml()), l or []), self.childs.values())
		return n
	def parse(self, node):
		"""
		Simple Text load
		"""
		self.valNode(node)
		self.value = str("").join([x.nodeValue.strip() for x in node.childNodes if x.nodeType == x.TEXT_NODE])
		self.attrs = dict(node.attributes.items())
		self.childs = dict ()
		def _parse(elt):
			o = parser(elt)(xml = elt)
			self.childs.setdefault(o._name, [])
			self.childs[o._name].append(o)

		map(_parse, filter(lambda x: x.nodeType == x.ELEMENT_NODE, node.childNodes))
                map(lambda x: setattr(self, x, self.attrs.get(x, None)), self._attr)

	def loadElt(self, elt):
		n = self.childs.pop(elt, None)
		if n and len(n) > 0:
			setattr(self, elt, n[0])

	def saveElt(self, elt):
		self.childs[elt] = [getattr(self, elt)]
	
	def valNode(self, node):
		if node.localName != self._name or node.namespaceURI != self._ns:
			raise RuntimeError("%s:%s node expected, got %s:%s" % (self._ns, self._name, node.namespaceURI, node.localName))

	
	def Element(self, tag, text = None):
		n = minidom.Element("%s:%s" % (self._prefix, tag), self._ns, prefix = self._prefix)
		if text is not None:
			t = minidom.Text()
			t.data = text
			n.appendChild(t)
		return n

class Undef(xBase):
	_name = None
	_ns = None
	def parse(self, node):
		self.node = node.cloneNode(deep = True)
	def xml(self):
		return self.node

class Detail(xBase):
	_name = "detail"

class Software(xBase):
	_name = "software"

class Hardware(xBase):
	_name = "hardware"

class Param(xBase):
	_name = "param"
	def name(self):
		return self.attrs.get("name", None)
map (parserAdd, [Detail, Software, Hardware, Param])

class Element(xBase):
	detail = None
	tests = []
	_name = "element"
	_attr = ["ns", "uuid"]

	def xml(self):
		self.saveElt("detail")
		self.childs['tests'] = self.tests
		n = xBase.xml(self)
		return n
	def parse(self, node):
		xBase.parse(self, node)
		self.loadElt("detail")
		self.tests = self.childs.pop("test", [])

	def run(self, tests, **kw):
		kw.setdefault("_element", self)
		xf = Flow()
                xf.uuid = self.attrs.get("uuid", "undefuned")
                if self.ns == "local":
                        h = kw.get("_host", None)
                        if not h:
                                raise RuntimeError("Element %s in local namespace but without owner host" % xf.uuid)
                        xf.uuid = "%s/%s" % (h.uuid, xf.uuid)
		xf.states = map (lambda t: t.run(tests, **kw), self.tests)
		xf.code = foldl(lambda i, s: min(i, s.code), 1, xf.states)
		return xf
parserAdd(Element)

class HostInfo(xBase):
	detail = None
	elements = []
	_name = "host"
	_attr = ["uuid"]

	def xml(self):
		self.saveElt("detail")
		self.childs['element'] = self.elements
		n = xBase.xml(self)
		return n

	def parse(self, node):
		xBase.parse(self, node)
		self.loadElt("detail")
		self.elements = self.childs.pop("element", [])

	def run(self, tests):
		kw = {"_host":self}
		return [e.run(tests, **kw) for e in self.elements]
parserAdd(HostInfo)

class HostMap(xBase):
	detail = None
	hosts = []
	_name = "hostmap"

	def xml(self):
		self.saveElt("detail")
		self.childs['host'] = self.hosts
		return xBase.xml(self)

	def parse(self, node):
		xBase.parse(self, node)
		self.loadElt("detail")
		self.hosts = self.childs.pop("host", [])
parserAdd(HostMap)

class StateMap(xBase):
	flows = []
	_name = "statemap"

	def xml(self):
		self.childs['flow'] = self.flows
		return xBase.xml(self)

	def parse(self, node):
		xBase.parse(self, node)
		self.flows = self.childs.pop("flow", [])
parserAdd(StateMap)

class State(xBase):
	test = None
	_name = "state"
        _attr = ['test', 'code']

	def xml(self):
		self.attrs['state'] = states.decode(self.code)
		return xBase.xml(self)
	def parse(self, node):
		xBase.parse(self, node)
parserAdd(State)

class Flow(xBase):
	origin = None
	states = []
	state = 0
	_name = "flow"
        _attr = ["uuid", "code"]

	def xml(self):
		self.attrs['state'] = states.decode(self.code)
		self.childs['state'] = self.states
		return xBase.xml(self)
	def parse(self, node):
		xBase.parse(self, node)
                if self.code is None: self.code = 0
		self.states = self.childs.pop("state", [])
parserAdd(Flow)

class Test(xBase):
	param = []
	_name = 'test'
        _attr = ['name']

	def xml(self):
		self.childs['param'] = self.param
		n = xBase.xml(self)
		return n

	def parse(self, node):
		xBase.parse(self, node)
		self.param = self.childs.pop("param", [])

	def run(self, tests, **kw):
		map(lambda p: kw.setdefault(str(p.name()), str(p.value)), self.param)
		xs = State()
		xs.test = self.name
		print kw
		(xs.code, xs.value) = tests.run(self.name, **kw)
		return xs
		
parserAdd(Test)
